﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using STFXPdfDotNET;
using System.IO;
using System.Diagnostics;
using STFXmlParser;
using Selections;
using CTNet;
using PdfDocument;

namespace CalcHelper
{
    public partial class Form1 : Form
    {
        public Form1(string[] args)
        {
            InitializeComponent();

            // Somehow this label got 'lost', so I need to do this.
            docLabel.Parent = tableLayoutPanel1;

            // If I collapse this in the designer I do not see the table layout panel
            // on the bottom with it's doc and page labels.
            splitContainer1.Panel2Collapsed = true;

            // Initialize the Box Number Magnification tool tip.
            this.boxNumMagToolTip = new System.Windows.Forms.ToolTip();
            this.boxNumMagToolTip.AutoPopDelay = 5000;
            this.boxNumMagToolTip.InitialDelay = 100;
            this.boxNumMagToolTip.OwnerDraw = true;
            this.boxNumMagToolTip.IsBalloon = false;
            this.boxNumMagToolTip.ReshowDelay = 10;
            this.boxNumMagToolTip.Active = false;
            this.boxNumMagToolTip.Draw += new DrawToolTipEventHandler(this.boxNumMagToolTip_Draw);
            this.boxNumMagToolTip.Popup += new PopupEventHandler(boxNumMagToolTip_Popup);
        }

        /*
         * The following icons: crossCircle.png, exclamation.png, and
         * information.png were downloaded from http://p.yusukekamiyamane.com/,
         * These icons are stored under 'Resources'.
        */

        /*
         * Current split screen implementation is as follows:
         * 
         * In single screen mode we use just pdf1 and pa1.
         * 
         * In split screen mode we add pdf2 and pa2 for the second screen panel.
         * We always refer to pdf1/pa1 as the left or top panel, pdf2/pa2 are
         * the right/bottom panel - our screen can be split either horizontally
         * or vertically.
         * 
         * Splitting the screen will automatically display the pdf1 pdf page in the
         * pdf2 panel.
         * 
         * When a document is opened for the pdf1 panel it will automatically be
         * displayed in the pdf2 panel if it is enabled.
         * 
         * The pdf1 panel has the screen splitting buttons. Clicking on the single
         * screen menu button will collapse the pdf2 panel so you will only see
         * what the pdf1 panel is displaying.
        */

        private void Form1_Resize(object sender, EventArgs e)
        {
            //MessageBox.Show("Form1_Resize!");

            /*
             * minMaxRestore is only set to true in WndProc() when
             * we detect a Minimize, Maximize, or Restore event.
            */
            if (pdf1 != null)
            {
                if (minMaxRestore)
                {
                    ProcessMinMaxRestore(ref pdf1, ref pa1);
                }
                else
                {
                    ProcessResize(ref pdf1, ref pa1);
                }
            }

            if (pdf2 != null)
            {
                if (minMaxRestore)
                {
                    ProcessMinMaxRestore(ref pdf2, ref pa2);
                }
                else
                {
                    ProcessResize(ref pdf2, ref pa2);
                }
            }

            if (minMaxRestore)
            {
                minMaxRestore = false;
            }
        }

        private void fitInWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf1, ref pa1);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Remove previously extracted files and the directory that holds them.
            if (docFile != null)
            {
                docFile.CleanupFiles();
            }

            // Save location and size.
            // Got this from CodeProject, can not recall the author.
            if (WindowState == FormWindowState.Maximized)
            {
                Properties.Settings.Default.Location = RestoreBounds.Location;
                Properties.Settings.Default.Size = RestoreBounds.Size;
                Properties.Settings.Default.Maximized = true;
                Properties.Settings.Default.Minimized = false;
            }
            else if (WindowState == FormWindowState.Normal)
            {
                Properties.Settings.Default.Location = Location;
                Properties.Settings.Default.Size = Size;
                Properties.Settings.Default.Maximized = false;
                Properties.Settings.Default.Minimized = false;
            }
            else
            {
                Properties.Settings.Default.Location = RestoreBounds.Location;
                Properties.Settings.Default.Size = RestoreBounds.Size;
                Properties.Settings.Default.Maximized = false;
                Properties.Settings.Default.Minimized = true;
            }

            Properties.Settings.Default.Save();

            // Disconnect from Calc Tool Server:
            if (ctClient != null)
            {
                ctClient.StopClient();
            }

            // Dispose of stuff.
            if (pdf1 != null)
            {
                pdf1.ClearBmaps();
            }

            // Dispose of more stuff.
            if (pdf2 != null)
            {
                pdf2.ClearBmaps();
            }
        }

        private void nextPageBtn_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf1, ref pa1);
        }

        private void prevPageBtn_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf1, ref pa1);
        }

        private void firstPageBtn_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf1, ref pa1);
        }

        private void lastPageBtn_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf1, ref pa1);
        }

        private void fullWidthBtn_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf1, ref pa1);
        }

        private void fullPageBtn_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf1, ref pa1);
        }

        protected override void WndProc(ref Message m)
        {
            if ((pdf1 != null) || (pdf2 != null))
            {
                // WM_SYSCOMMAND?
                if (m.Msg == 0x0112) 
                {
                    int wParam = (m.WParam.ToInt32() & 0xFFF0);
                    bool visible = true;

                    /*
                     * 0xF020 Minimize event - SC_MINIMIZE from Winuser.h
                     * 0xF030 Maximize event - SC_MAXIMIZE from Winuser.h
                     * 0xF120 Restore event - SC_RESTORE from Winuser.h
                    */
                    if ((wParam == 0xF020) || (wParam == 0xF030) || (wParam == 0xF120))
                    {
                        minMaxRestore = true;

                        /*
                         * Check if window WILL be maximized. If so, set
                         * pictureBox1.visible to false so they do not see it
                         * flash on screen before being rescaled and re-displayed
                         * in the Resize event handler. This is the only way I
                         * could find to do this.
                        */
                        if (wParam == 0xF030)
                        {
                            visible = false;
                        }

                        if (pdf1 != null)
                        {
                            pdf1.panel.Visible = visible;
                        }

                        if (pdf2 != null)
                        {
                            pdf2.panel.Visible = visible;
                        }
                    }
                }
            }

            base.WndProc(ref m);
        }

        private void zoomOutBtn_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf1, ref pa1);
        }

        private void zoomInBtn_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf1, ref pa1);
        }

        private void zoomResetBtn_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf1, ref pa1);
        }

        private void pageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav != Pdf.pageNavMode.page)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.page, ref pdf1, ref pa1);
            }
        }

        private void boxedPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav != Pdf.pageNavMode.boxedpages)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.boxedpages, ref pdf1, ref pa1);
            }
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if (pdf1 == null)
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox1_MouseDown():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse events can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            //MessageBox.Show(string.Format("MouseDown"));

            // We only do left mouse clicks when in Box Number Magnification mode.
            if ((e.Button == System.Windows.Forms.MouseButtons.Left) && (ctClient.boxSelClient != boxSelDest.boxNumMag))
            {
                ProcessLeftMouseDown(ref pdf1, e.Location);
            }
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if (pdf1 == null)
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox1_MouseMove():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse events can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // We only do left mouse clicks when in Box Number Magnification mode.
            if ((e.Button == System.Windows.Forms.MouseButtons.Left) && (ctClient.boxSelClient != boxSelDest.boxNumMag))
            {
                ProcessLeftMouseMove(ref pdf1, e.Location);
            }
        }

        private void pictureBox1_MouseClick(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if ((pdf1 == null) || (pa1 == null))
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox1_MouseClick():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse clicks can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            /*
             * Now determine if a left or right mouse button click as
             * they perform different functions.
             */
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                if (ctClient.boxSelClient == boxSelDest.boxNumMag)
                {
                    ProcessLeftMouseBoxNumMagClick(ref pdf1, ref pa1, e.Location);
                }
                else if (pdf1.mLassoMove == false)
                {
                    ProcessLeftMouseClick(ref pdf1, ref pa1, e.Location);
                }
            }
            else if (e.Button == System.Windows.Forms.MouseButtons.Right)
            {
                if (ctClient.boxSelClient == boxSelDest.boxNumMag)
                {
                    /*
                     * Box Number Magnification locks out all other mouse clicks:
                     */
                    msg = "Right Mouse Clicks Are Disabled:\n\nThe box selection mode 'Box Number Magnification' locks out right mouse clicks.\n\nRight mouse click will be ignored.";
                }
                else if (ctClient.vscodeLock == true)
                {
                    /*
                     * No sense in going further if the document is locked:
                     */
                    msg = string.Format("Right Mouse Clicks Are Disabled:\n\nThe Visual Studio Code file that Calc Helper launched from:\n\n{0}\n\nIs no longer the active Visual Studio Code file.", ctClient.vscFile);
                }

                if (msg.Length > 0)
                {
                    using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                    {
                        mainMsg.ShowDialog(this);
                    }
                }
                else
                {
                    ProcessRightMouseClick(ref pdf1, ref pa1, e.Location);
                }
            }
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if ((pdf1 == null) || (pa1 == null))
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox1_MouseUp():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse events can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // We only do left mouse clicks when in Box Number Magnification mode.
            if ((e.Button == System.Windows.Forms.MouseButtons.Left) && (ctClient.boxSelClient != boxSelDest.boxNumMag))
            {
                ProcessLeftMouseUp(ref pdf1, ref pa1);
            }
        }

        public void NavToPageNum(ref Pdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "\nNavToPageNum() received a null pdf argument."))
                {
                    mainMsg.ShowDialog(this);
                }
                return;
            }

            if (pa == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "\nNavToPageNum() received a null pa argument."))
                {
                    mainMsg.ShowDialog(this);
                }
                return;
            }

            using (var goToPage = new GoToPage(pdf.numPages))
            {
                // Could be displaying the Calc Tool window.
                //goToPage.TopMost = true;

                var result = goToPage.ShowDialog(this);

                if (result == DialogResult.OK)
                {
                    pdf.curPage = goToPage.selPage;

                    pdf.ShowPage(ref docFile.hPdf, (int)pdf.curPage, true, ref pa);

                    ProcessPageNum(ref pdf);
                }
            }
        }

        public void NavToBoxedPageNum(ref Pdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "\nNavToBoxedPageNum() received a null pdf argument."))
                {
                    mainMsg.ShowDialog(this);
                }
                return;
            }

            if (pa == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "\nNavToBoxedPageNum() received a null pa argument."))
                {
                    mainMsg.ShowDialog(this);
                }
                return;
            }

            using (var goToBoxedPage = new GoToBoxedPage(pdf.pageNav, ref pdf.boxedPageList))
            {
                goToBoxedPage.TopMost = true;

                var result = goToBoxedPage.ShowDialog(this);

                if (result == DialogResult.OK)
                {
                    pdf.curPage = goToBoxedPage.selBoxedPage;
                    pdf.boxedPageIndex = pdf.boxedPageList.IndexOf(pdf.curPage);

                    pdf.ShowPage(ref docFile.hPdf, (int)pdf.curPage, true, ref pa);

                    ProcessPageNum(ref pdf);
                }
            }
        }

        private void goToPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav == Pdf.pageNavMode.page)
            {
                NavToPageNum(ref pdf1, ref pa1);
                //MessageBox.Show(string.Format("Back from F5 - Page Nav Mode: {0}", pdf1.curPage));
            }
            else
            {
                NavToBoxedPageNum(ref pdf1, ref pa1);
                //MessageBox.Show(string.Format("Back from F5 - Boxed Page Nav Mode: {0}", pdf1.curPage));
            }
        }

        // Had to do this in order to set the 'View' menus shortcut combinations.
        protected override bool ProcessCmdKey(ref Message msg, Keys keys)
        {
            //MessageBox.Show("Entered ProcessCmdKey()");

            switch (keys)
            {
                case (Keys.Control | Keys.Add):
                case (Keys.Control | Keys.Oemplus):

                    //MessageBox.Show("Ctrl++");

                    if (zoomInBtn.Enabled)
                    {
                        ProcessZoomIn(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.Add):
                case (Keys.Alt | Keys.Oemplus):

                    //MessageBox.Show("Alt++");

                    if (zoomInBtn2.Enabled)
                    {
                        ProcessZoomIn(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.Subtract):
                case (Keys.Control | Keys.OemMinus):

                    //MessageBox.Show("Ctrl+-");

                    if (zoomOutBtn.Enabled)
                    {
                        ProcessZoomOut(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.Subtract):
                case (Keys.Alt | Keys.OemMinus):

                    //MessageBox.Show("Alt+-");

                    if (zoomOutBtn2.Enabled)
                    {
                        ProcessZoomOut(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.NumPad0):

                    //MessageBox.Show("Ctrl+0");

                    if (zoomResetBtn.Enabled)
                    {
                        ProcessZoomReset(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.NumPad0):

                    //MessageBox.Show("Alt+0");

                    if (zoomResetBtn2.Enabled)
                    {
                        ProcessZoomReset(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.Home):

                    //MessageBox.Show("Ctrl+Home");

                    if (firstPageBtn.Enabled)
                    {
                        ProcessFirstPageNav(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.Home):

                    //MessageBox.Show("Alt+Home");

                    if (firstPageBtn2.Enabled)
                    {
                        ProcessFirstPageNav(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.PageDown):

                    //MessageBox.Show("Ctrl+Page Down");

                    if (nextPageBtn.Enabled)
                    {
                        ProcessNextPageNav(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.PageDown):

                    //MessageBox.Show("Alt+Page Down");

                    if (nextPageBtn2.Enabled)
                    {
                        ProcessNextPageNav(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.PageUp):

                    //MessageBox.Show("Ctrl+Page Up");

                    if (prevPageBtn.Enabled)
                    {
                        ProcessPrevPageNav(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.PageUp):

                    //MessageBox.Show("Alt+Page Up");

                    if (prevPageBtn2.Enabled)
                    {
                        ProcessPrevPageNav(ref pdf2, ref pa2);
                    }
                    break;

                default:
                    
                    return base.ProcessCmdKey(ref msg, keys);
            }

            return true;
        }

        private void firstPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf1, ref pa1);
        }

        private void previousPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf1, ref pa1);
        }

        private void nextPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf1, ref pa1);
        }

        private void lastPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf1, ref pa1);
        }

        private void zoomInToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf1, ref pa1);
        }

        private void zoomOutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf1, ref pa1);
        }

        private void zoomResetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf1, ref pa1);
        }

        private void fullWidthToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf1, ref pa1);
        }

        private void singleScrnBtn_Click(object sender, EventArgs e)
        {
            ProcessSingleScrnCLick();
        }

        private void splitHorizBtn_Click(object sender, EventArgs e)
        {
            ProcessSplitHorScrnCLick();
        }

        private void splitVertBtn_Click(object sender, EventArgs e)
        {
            ProcessSplitVerScrnCLick();
        }

        private void splitContainer1_SplitterMoving(object sender, SplitterCancelEventArgs e)
        {
            splitterMoving = true;
        }

        private void splitContainer1_SplitterMoved(object sender, SplitterEventArgs e)
        {
            if (splitterMoving)
            {
                /*
                 * This event was triggered when someone stopped moving the
                 * splitter bar as opposed to being triggered due to a form resize
                 * event which ends up calling ProcessResize() too.
                 * 
                 * We need to check both pdf's for null because we can have a split
                 * screen with both panels empty.
                */
                if (pdf1 != null)
                {
                    ProcessResize(ref pdf1, ref pa1);
                }

                if (pdf2 != null)
                {
                    ProcessResize(ref pdf2, ref pa2);
                }

                splitterMoving = false;
            }
        }

        private void firstPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf2, ref pa2);
        }

        private void previousPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf2, ref pa2);
        }

        private void nextPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf2, ref pa2);
        }

        private void lastPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf2, ref pa2);
        }

        private void goToPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav == Pdf.pageNavMode.page)
            {
                NavToPageNum(ref pdf2, ref pa2);
                //MessageBox.Show(string.Format("Back from F5 - Page Nav Mode: {0}", pdf2.curPage));
            }
            else
            {
                NavToBoxedPageNum(ref pdf2, ref pa2);
                //MessageBox.Show(string.Format("Back from F5 - Boxed Page Nav Mode: {0}", pdf2.curPage));
            }
        }

        private void zoomInToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf2, ref pa2);
        }

        private void zoomOutToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf2, ref pa2);
        }

        private void zoomResetToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf2, ref pa2);
        }

        private void fullWidthToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf2, ref pa2);
        }

        private void fullPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf2, ref pa2);
        }

        private void firstPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf2, ref pa2);
        }

        private void prevPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf2, ref pa2);
        }

        private void nextPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf2, ref pa2);
        }

        private void lastPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf2, ref pa2);
        }

        private void pageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav != Pdf.pageNavMode.page)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.page, ref pdf2, ref pa2);
            }
        }

        private void boxedPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav != Pdf.pageNavMode.boxedpages)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.boxedpages, ref pdf2, ref pa2);
            }
        }

        private void zoomInBtn2_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf2, ref pa2);
        }

        private void zoomOutBtn2_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf2, ref pa2);
        }

        private void zoomResetBtn2_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf2, ref pa2);
        }

        private void fullWidthBtn2_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf2, ref pa2);
        }

        private void fullPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf2, ref pa2);
        }

        private void singleScreenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessSingleScrnCLick();
        }

        private void splitScreenHorizontallyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessSplitHorScrnCLick();
        }

        private void splitScreenVerticallyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessSplitVerScrnCLick();
        }

        private void pictureBox2_MouseDown(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if (pdf2 == null)
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox2_MouseDown():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse events can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            //MessageBox.Show(string.Format("MouseDown"));

            // We only do left mouse clicks when in Box Number Magnification mode.
            if ((e.Button == System.Windows.Forms.MouseButtons.Left) && (ctClient.boxSelClient != boxSelDest.boxNumMag))
            {
                ProcessLeftMouseDown(ref pdf2, e.Location);
            }
        }

        private void pictureBox2_MouseMove(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if (pdf2 == null)
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox2_MouseMove():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse events can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // We only do left mouse clicks when in Box Number Magnification mode.
            if ((e.Button == System.Windows.Forms.MouseButtons.Left) && (ctClient.boxSelClient != boxSelDest.boxNumMag))
            {
                ProcessLeftMouseMove(ref pdf2, e.Location);
            }
        }

        private void pictureBox2_MouseClick(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if ((pdf2 == null) || (pa2 == null))
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox2_MouseClick():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse clicks can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            /*
             * Now determine if a left or right mouse button click as
             * they perform different functions.
             */
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                if (ctClient.boxSelClient == boxSelDest.boxNumMag)
                {
                    ProcessLeftMouseBoxNumMagClick(ref pdf2, ref pa2, e.Location);
                }
                else if (pdf2.mLassoMove == false)
                {
                    ProcessLeftMouseClick(ref pdf2, ref pa2, e.Location);
                }
            }
            else if (e.Button == System.Windows.Forms.MouseButtons.Right)
            {
                if (ctClient.boxSelClient == boxSelDest.boxNumMag)
                {
                    /*
                     * Box Number Magnification locks out all other mouse clicks:
                     */
                    msg = "Right Mouse Clicks Are Disabled:\n\nThe box selection mode 'Box Number Magnification' locks out right mouse clicks.\n\nRight mouse click will be ignored.";
                }
                else if (ctClient.vscodeLock == true)
                {
                    /*
                     * No sense in going further if the document is locked:
                     */
                    msg = string.Format("Right Mouse Clicks Are Disabled:\n\nThe Visual Studio Code file that Calc Helper launched from:\n\n{0}\n\nIs no longer the active Visual Studio Code file.", ctClient.vscFile);
                }

                if (msg.Length > 0)
                {
                    using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                    {
                        mainMsg.ShowDialog(this);
                    }
                }
                else
                {
                    ProcessRightMouseClick(ref pdf2, ref pa2, e.Location);
                }
            }
        }

        private void pictureBox2_MouseUp(object sender, MouseEventArgs e)
        {
            string msg = "";

            // Being cautious....
            if ((pdf2 == null) || (pa2 == null))
            {
                return;
            }

            if (ctClient == null)
            {
                msg = "pictureBox2_MouseUp():\n\nctClient is null so Calc Helper cannot communicate with the Calc Tool Server.\n\nMouse events can not be processed.";

                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, msg))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // We only do left mouse clicks when in Box Number Magnification mode.
            if ((e.Button == System.Windows.Forms.MouseButtons.Left) && (ctClient.boxSelClient != boxSelDest.boxNumMag))
            {
                ProcessLeftMouseUp(ref pdf2, ref pa2);
            }
        }

        private void ToolBeltClosed(object sender, FormClosedEventArgs e)
        {
            // MIGHT not be using this...will set the box selection buffer
            // with selected boxes [will be moved to what is now called ProcessCalc.cs]
            // Needs more thought.
            Form1.BoxSel -= toolBelt.Form1_CalcToolBoxSel;

            /*
             * Unsubscribing from Selections (ProcessSelections.cs)
             * unselection event.
             */
            BoxSelections.BoxUnSelEvent -= toolBelt.BoxSelections_UnSel;
            
            ToolBelt.ToolBeltEvent -= toolBelt_ToolBeltEvent;

            // Not sure about needing this dispose, being cautious.
            toolBelt.Dispose();

            toolBelt = null;

            if (docFile != null)
            {
                docFile.tb = null;
            }

            /*
             * Check the Tool Belt's 'operator wait boxes' list to see if it
             * contains any boxes. This list is a holding area as the Tool Belt
             * 'waits' for the user to choose an operator to apply to the boxes
             * before it can send them on to the editor.
             */
            if (boxSel.toolBeltOpWaitBoxes.Count > 0)
            {
                if (!Pa1OrPdf1AreNull("ToolBeltClosed", "Selection resets will not be possible."))
                {
                    boxSel.ResetSelections(boxSelTypes.leftToolBeltOpWaits, ref pa1, ref pdf1, ref pa2, ref pdf2);
                    RefreshPanels();
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Restore location and size.
            // Got this from CodeProject, can not recall the author.
            if (Properties.Settings.Default.Maximized)
            {
                Location = Properties.Settings.Default.Location;
                WindowState = FormWindowState.Maximized;
                Size = Properties.Settings.Default.Size;
            }
            else if (Properties.Settings.Default.Minimized)
            {
                Location = Properties.Settings.Default.Location;
                WindowState = FormWindowState.Minimized;
                Size = Properties.Settings.Default.Size;
            }
            else
            {
                Location = Properties.Settings.Default.Location;
                Size = Properties.Settings.Default.Size;
            }

            /*
             * We need to do this here because we can not call
             * Application.Exit() from a [critical] ctClient exception
             * or error condition during ctClient.StartClient() when 
             * ctClient.StartClient() is called from the Form1 constructor
             * as we get a disposed object exception since the Form1 will
             * be attempting to open/accessed while it is already disposed of.
             * 
             * We have always had to perform the .Visible thing also,
             * else you would necver see the form.....strange.
             */
            //this.Visible = true;

            ctClient = new CTClient(vscFileInfo);

            /*
             * Subscribe to ctClient's event so we can be informed concerning
             * communication with the calc tool server as it relates to the
             * Calc Helper tool.
             */
            ctClient.CtClientEvent += ctClient_CtClientEvent;

            ctClient.StartClient();
        }

        /*********************** Box Selection Event Setup Start ***********************/

        // A custom EventArgs class to pass selected boxes to the Tool Belt Box Selector.
        public class BoxSelArgs : EventArgs
        {
            public string cmd { get; set; }
            public List<string> boxes { get; set; }
        }

        // Create the box selection event itself:
        public static event EventHandler<BoxSelArgs> BoxSel;

        // Trigger the box selection event:
        protected virtual void OnBoxSel(BoxSelArgs e)
        {
            if (BoxSel != null)
            {
                BoxSel(this, e);
            }
        }

        /*********************** Box Selection Event Setup Stop ***********************/

        // Calc Tool Client event handler:
        void ctClient_CtClientEvent(object sender, CtClientArgs e)
        {
            if (e != null)
            {
                switch (e.eventType)
                {
                    case (ctEvent.cbcBoxSelReq):

                        /*
                         * Checkbox Control has successfully requested checkbox selections.
                         * We need to tell the Dataflow Retriever tool that it will no longer
                         * be receiving box selections IF it is curretly receiving them, and
                         * we need to do some related processing.
                         */
                        if (ctClient.boxSelClient == boxSelDest.dfRetriever)
                        {
                            ProcessDFRetMode(false, true, false);
                        }

                        ProcessChkBoxCtrlMode(true, false, true);
                        break;

                    case (ctEvent.cbcBoxSets):

                        /*
                         * Checkbox Control has sent Calc Helper the current Checkbox Sets
                         * which will be processed by Calc Helper and the resulting Checkbox
                         * Control adds/deletes will be displayed for the user. This helps to
                         * give the user visual feedback regarding their checkbox selections.
                         */
                        if (docFile == null)
                        {
                            this.Focus();

                            using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null docFile:\n\nctClient_CtClientEvent() has detected a null docFile while processing Checkbox Sets from Checkbox Control.\n\nProcessing  Will Be Aborted."))
                            {
                                mainMsg.ShowDialog(this);
                            }

                            return;
                        }

                        if (pa1 == null)
                        {
                            this.Focus();

                            using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null pa1:\n\nctClient_CtClientEvent() has detected a null pa1 while processing Checkbox Sets from Checkbox Control.\n\nProcessing  Will Be Aborted."))
                            {
                                mainMsg.ShowDialog(this);
                            }

                            return;
                        }

                        if (pdf1 == null)
                        {
                            this.Focus();

                            using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null pdf1:\n\nctClient_CtClientEvent() has detected a null pdf1 while processing Checkbox Sets from Checkbox Control.\n\nProcessing  Will Be Aborted."))
                            {
                                mainMsg.ShowDialog(this);
                            }

                            return;
                        }

                        if (docFile.ProcessChkBoxCtrlSets(e.eventMsg, ref pa1, ref pdf1, ref pa2, ref pdf2) == true)
                        {
                            RefreshPanels();
                        }

                        break;

                    case (ctEvent.cbcBoxUnSel):

                        /*
                         * Checkbox Control has sent Calc Helper one or more boxes to
                         * unselect.
                         */
                        if (docFile == null)
                        {
                            this.Focus();

                            using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null docFile:\n\nctClient_CtClientEvent() has detected a null docFile while processing Checkbox Unselections from Checkbox Control.\n\nProcessing  Will Be Aborted."))
                            {
                                mainMsg.ShowDialog(this);
                            }

                            return;
                        }

                        if (pa1 == null)
                        {
                            this.Focus();

                            using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null pa1:\n\nctClient_CtClientEvent() has detected a null pa1 while processing Checkbox Unselections from Checkbox Control.\n\nProcessing  Will Be Aborted."))
                            {
                                mainMsg.ShowDialog(this);
                            }

                            return;
                        }

                        if (pdf1 == null)
                        {
                            this.Focus();

                            using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null pdf1:\n\nctClient_CtClientEvent() has detected a null pdf1 while processing Checkbox Unselections from Checkbox Control.\n\nProcessing  Will Be Aborted."))
                            {
                                mainMsg.ShowDialog(this);
                            }

                            return;
                        }

                        char[] comma = new char[] { ',' };
                        boxSel.unselLeftBoxes = e.eventMsg.Split(comma, StringSplitOptions.RemoveEmptyEntries).ToList();
                        boxSel.ResetSelections(boxSelTypes.leftSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                        RefreshPanels();

                        break;

                    case (ctEvent.cbcBoxSetsFileNack):

                        /*
                         * Checkbox Control has sent Calc Helper the current Checkbox Sets
                         * but Checkbox Control is linked to a different file, so we will
                         * inform the user and ignore the boxes.
                         */
                        this.Focus();

                        using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, e.eventMsg))
                        {
                            mainMsg.ShowDialog(this);
                        }

                        break;

                    case (ctEvent.dfrBoxSelReq):

                        /*
                         * Dataflow Retriever has successfully requested box selections.
                         * We need to tell the Checkbox Control tool that it will no longer
                         * be receiving box selections IF it is curretly receiving them, and
                         * we need to do some related processing.
                         */
                        if (ctClient.boxSelClient == boxSelDest.chkBoxCtrl)
                        {
                            ProcessChkBoxCtrlMode(false, true, false);
                        }

                        ProcessDFRetMode(true, false, true);
                        break;

                    case (ctEvent.boxSelOff):

                        /*
                         * A tool no longer wants box selections.
                         */
                        switch (ctClient.boxSelClient)
                        {
                            case (boxSelDest.chkBoxCtrl):

                                ProcessChkBoxCtrlMode(false, false, true);
                                break;

                            case (boxSelDest.dfRetriever):

                                ProcessDFRetMode(false, false, true);
                                break;
                        }

                        break;

                    case (ctEvent.connected):

                        // Calc Helper has connected to Calc Tool Server.
                        isConnected = true;
                        vsCodeIconLbl.Image = CalcHelper.Properties.Resources.VscodeGreen;
                        vsCodeIconLbl.ToolTipText = "Connected To Vscode File";
                        boxSel = new BoxSelections();
                        OpenFile(ctClient.zipFile, ctClient.vscFile);

                        // Process the Document Box Selection Mode setting.
                        if (Properties.Settings.Default.DocBoxSelMode == true)
                        {
                            docBoxSelModeAtStartupToolStripMenuItem.Text = "Disable 'Document Box Selection Mode' At Startup";
                            docBoxSelToolStripMenuItem.PerformClick();
                        }
                        else
                        {
                            docBoxSelModeAtStartupToolStripMenuItem.Text = "Enable 'Document Box Selection Mode' At Startup";
                        }

                        /*
                         * ----------------------------------------------
                         * Create debug file:
                         */
                        //using (StreamWriter outputFile = new StreamWriter(debugFile))
                        //{
                        //    outputFile.WriteLine("Debuging:");
                        //}
                        /*
                         * ----------------------
                         * 
                         * Example of writing to it:
                         */
                        //using (StreamWriter outputFile = new StreamWriter(debugFile, true))
                        //{
                        //    outputFile.WriteLine(string.Format("Entering-navMode: {0}", navMode.ToString()));
                        //}
                        /*
                         * ----------------------------------------------
                         * 
                         */

                        break;

                    case (ctEvent.connectFail):

                        //MessageBox.Show("ctClient_CtClientEvent - ctEvent.connectFail", "Calc Helper-StartClient():", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        Application.Exit();
                        break;

                    case (ctEvent.shutdown):

                        /* 
                         * Code editor is going down, so we will close this application too.
                         */
                        Application.Exit();
                        break;

                    case (ctEvent.saveDoc):

                        /* 
                         * Let user know they can 'refresh' Calc Helper with info in
                         * the Vscode JS file.
                         */
                        refreshBtn.Enabled = true;
                        refreshBtn.Image = CalcHelper.Properties.Resources.RefreshOn;
                        break;

                    case (ctEvent.saveUnlockDoc):

                        /* 
                         * A strange case, we were in lock mode from a previous event,
                         * then got a save msg from vscode for our linked file. Let
                         * user know they can 'refresh' Calc Helper with info in the
                         * Vscode JS file.
                         */
                        ProcessDocLock(false);
                        refreshBtn.Enabled = true;
                        refreshBtn.Image = CalcHelper.Properties.Resources.RefreshOn;
                        break;

                    case (ctEvent.docLock):

                        /* 
                         * Lost file congruency with Vscode. Going into lockdown.
                         */
                        ProcessDocLock(true);

                        this.Focus();

                        using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.warning, e.eventMsg))
                        {
                            mainMsg.ShowDialog(this);
                        }

                        break;

                    case (ctEvent.docUnLock):

                        /* 
                         * Regained file congruency with Vscode.
                         */
                        ProcessDocLock(false);

                        break;

                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void chkboxPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav != Pdf.pageNavMode.checkbox)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.checkbox, ref pdf1, ref pa1);
            }
        }

        private void code39ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav != Pdf.pageNavMode.code39)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.code39, ref pdf1, ref pa1);
            }
        }

        private void int2of5ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav != Pdf.pageNavMode.int2of5)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.int2of5, ref pdf1, ref pa1);
            }
        }

        private void pdf417ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav != Pdf.pageNavMode.pdf417)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.pdf417, ref pdf1, ref pa1);
            }
        }

        private void qrCodesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav != Pdf.pageNavMode.qrcode)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.qrcode, ref pdf1, ref pa1);
            }
        }

        private void chkboxToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav != Pdf.pageNavMode.checkbox)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.checkbox, ref pdf2, ref pa2);
            }
        }

        private void code39ToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav != Pdf.pageNavMode.code39)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.code39, ref pdf2, ref pa2);
            }
        }

        private void int2of5ToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav != Pdf.pageNavMode.int2of5)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.int2of5, ref pdf2, ref pa2);
            }
        }

        private void pdf417ToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav != Pdf.pageNavMode.pdf417)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.pdf417, ref pdf2, ref pa2);
            }
        }

        private void qrCodesToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav != Pdf.pageNavMode.qrcode)
            {
                ProcessPageNavToolStripClick(Pdf.pageNavMode.qrcode, ref pdf2, ref pa2);
            }
        }

        private void boxSelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (toolBelt == null)
            {
                ToolBeltStartUp(ToolBelt.tbTabPage.boxSel);
            }
            else if (toolBelt.tabPage != ToolBelt.tbTabPage.boxSel)
            {
                toolBelt.SelectTabPage(ToolBelt.tbTabPage.boxSel);
            }
        }

        private void codeNavToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (toolBelt == null)
            {
                ToolBeltStartUp(ToolBelt.tbTabPage.codeNav);
            }
            else if (toolBelt.tabPage != ToolBelt.tbTabPage.codeNav)
            {
                toolBelt.SelectTabPage(ToolBelt.tbTabPage.codeNav);
            }
        }

        private void fcnPickToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (toolBelt == null)
            {
                ToolBeltStartUp(ToolBelt.tbTabPage.fcnPick);
            }
            else if (toolBelt.tabPage != ToolBelt.tbTabPage.fcnPick)
            {
                toolBelt.SelectTabPage(ToolBelt.tbTabPage.fcnPick);
            }
        }

        private void snipPickToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (toolBelt == null)
            {
                ToolBeltStartUp(ToolBelt.tbTabPage.snipPick);
            }
            else if (toolBelt.tabPage != ToolBelt.tbTabPage.snipPick)
            {
                toolBelt.SelectTabPage(ToolBelt.tbTabPage.snipPick);
            }
        }

        /*
         * The Refresh button turns green when the Javascript code file has
         * been saved. The Refresh button is enabled and set to the green
         * image.
         */
        private void refreshBtn_Click(object sender, EventArgs e)
        {
            refreshBtn.Image = CalcHelper.Properties.Resources.RefreshOff;
            refreshBtn.Enabled = false;

            if (docFile == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null docFile:\n\nrefreshBtn_Click() has detected a null docFile.\n\nRefresh will not be possible."))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // These pa1 and pdf1 cannot be null, else we abort this event.
            if (Pa1OrPdf1AreNull("refreshBtn_Click", "Refresh will not be possible.") == true)
            {
                return;
            }

            /*
             * 'true' indicates this is a Refresh, as opposed to 'we just opened
             * the file'. We pass the page array args to set the fields on the
             * pages where we will indicate any controlled boxes, like dataflow
             * or calcs for example.
             * 
             * We then reset all selections on the document, and refresh the panels
             * to reflect the now current controlled boxes and unselections.
             * 
             * Lastly, we set the Tool Belt's Code Navigator lists to reflect any
             * changes to it's lists.
             */
            if (docFile.ProcessJSFile(true, ref pa1, ref pdf1, ref pa2, ref pdf2) == true)
            {
                boxSel.ResetSelections(boxSelTypes.allSel, ref pa1, ref pdf1, ref pa2, ref pdf2);
                RefreshPanels();
                docFile.SetCodeNavLists();

                ProcessOrphanBoxesButton();
            }
        }

        /*
         * Clicked to either set box selection mode for the Document, i.e. boxes
         * are passed on to the editor for insertion at the cursor location, or
         * to reset the box selection mode to 'Off'.
         */
        private void docBoxSelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ctClient == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null ctClient:\n\ndocBoxSelToolStripMenuItem_Click() has detected a null ctClient.\n\nThe 'Document' Box Selection button click cannot be acted upon."))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // Determine what the current box selecion mode is:
            switch (ctClient.boxSelClient)
            {
                case (boxSelDest.chkBoxCtrl):

                    /*
                     * User wants to quit sending check box selections to the Checkbox
                     * Control tool and to instead send box selections to the Vscode file.
                     */
                    ProcessChkBoxCtrlMode(false, true, false);
                    ProcessBoxSelMode(boxSelDest.document);
                    break;

                case (boxSelDest.dfRetriever):

                    /*
                     * User wants to quit sending box selections to the Dataflow
                     * Retriever tool and to instead send box selections to the
                     * Vscode file.
                     */
                    ProcessDFRetMode(false, true, false);
                    ProcessBoxSelMode(boxSelDest.document);
                    break;

                case (boxSelDest.boxNumMag):

                    /*
                     * User wants to quit selecting boxes to see the box numbers
                     * in a tooltip popup and to instead send box selections to the
                     * Vscode file.
                     */
                    ProcessBoxSelMode(boxSelDest.document);
                    break;

                case (boxSelDest.off):

                    /*
                     * User wants to start sending box selections to the Vscode file.
                     */
                    ProcessBoxSelMode(boxSelDest.document);
                    break;

                default:

                    /*
                     * User wants to quit sending box selections to the Vscode file
                     * and we go to the 'Off' box selection mode.
                     */
                    ProcessBoxSelMode(boxSelDest.off);
                    break;
            }
        }

        /*
         * User can only disable this box selection mode. It is enabled only by a
         * checkbox selection request from the Checkbox Control tool.
         * 
         * ProcessChkBoxCtrlMode() Arguments:
         * 
         *  'false' - We are switching from 'Checkbox Control' box selection mode to
         *      'No Selection'.
         *      
         *  'true' - We need to tell ctClient to send the 'box selection off' message
         *      to the Checkbox Control tool so it can reset itself regarding check
         *      box selections.
         *      
         *  'true' - We will call ProcessBoxSelMode() to reflect the switch 'from'.
         */
        private void chkBoxCtrlToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessChkBoxCtrlMode(false, true, true);
        }

        /*
         * User can only disable this box selection mode. It is enabled only by a
         * box selection request from the Dataflow Retriever tool.
         * 
         * ProcessDFRetMode() Arguments:
         * 
         *  'false' - We are switching from 'Dataflow Retriever' box selection mode to
         *      'No Selection'.
         *      
         *  'true' - We need to tell ctClient to send the 'box selection off' message
         *      to the Dataflow Retriever tool so it can reset itself regarding box
         *      selections.
         *      
         *  'true' - We will call ProcessBoxSelMode() to reflect the switch 'from'.
         */
        private void dfRetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessDFRetMode(false, true, true);
        }

        /*
         * Clicked to either set box selection mode for viewing the box numbers
         * via a tooltip popup or to reset the box selection mode to 'Off'.
         * 
         * Doing it here as I am implementing this with left mouse clicks,
         * so we still have box selections going on, in this case the selected
         * box goes 'nowhere'. Doing this for hard to see box numbers like
         * checkboxes, and individually boxed characters.
         */
        private void boxNumMagToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ctClient == null)
            {
                using (MessageForm mainMsg = new MessageForm(MessageForm.msgIcon.error, "null ctClient:\n\nboxNumMagToolStripMenuItem_Click() has detected a null ctClient.\n\nThe 'Box Number Magnification' Box Selection button click cannot be acted upon."))
                {
                    mainMsg.ShowDialog(this);
                }

                return;
            }

            // Determine what the current box selecion mode is:
            switch (ctClient.boxSelClient)
            {
                case (boxSelDest.document):

                    /*
                     * User wants to quit sending box selections to the Vscode file
                     * and to instead select boxes for the purpose of seeing
                     * the box numbers magnified.
                     */
                    ProcessBoxSelMode(boxSelDest.boxNumMag);
                    break;

                case (boxSelDest.chkBoxCtrl):

                    /*
                     * User wants to quit sending check box selections to the Checkbox
                     * Control tool and to instead select boxes for the purpose of seeing
                     * the box numbers magnified.
                     */
                    ProcessChkBoxCtrlMode(false, true, false);
                    ProcessBoxSelMode(boxSelDest.boxNumMag);
                    break;

                case (boxSelDest.dfRetriever):

                    /*
                     * User wants to quit sending box selections to the Dataflow
                     * Retriever tool and to instead select boxes for the purpose of seeing
                     * the box numbers magnified.
                     */
                    ProcessDFRetMode(false, true, false);
                    ProcessBoxSelMode(boxSelDest.boxNumMag);
                    break;

                case (boxSelDest.off):

                    /*
                     * User wants to start selecting boxes for the purpose of seeing
                     * the box numbers magnified.
                     */
                    ProcessBoxSelMode(boxSelDest.boxNumMag);
                    break;

                default:

                    /*
                     * User wants to quit box selection magnification, so we will set
                     * the box selection mode to 'Off'.
                     */
                    ProcessBoxSelMode(boxSelDest.off);
                    break;
            }
        }

        /*
         * Allows the user to select Document Box Selection mode at strtup
         * or to have it disabled at startup.
         */
        private void docBoxSelModeAtStartupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.DocBoxSelMode == true)
            {
                // User wants to disable Document Box Selection mode at startup.
                ProcessDocBoxSelStartupMode(false);
            }
            else
            {
                // User wants to enable Document Box Selection mode at startup.
                ProcessDocBoxSelStartupMode(true);
            }
        }

        /*
         * ***************
         * Setting focus to the Main form from the ToolBelt: 
         */
        private void pictureBox1_MouseEnter(object sender, EventArgs e)
        {
            this.Focus();

            if ((ctClient != null) && (ctClient.boxSelClient == boxSelDest.boxNumMag))
            {
                this.boxNumMagToolTip.Active = true;

                // Remove a prior box num holdover.
                boxNumMagToolTip.SetToolTip(this.pictureBox1, "");
            }
        }

        private void pictureBox2_MouseEnter(object sender, EventArgs e)
        {
            this.Focus();

            if ((ctClient != null) && (ctClient.boxSelClient == boxSelDest.boxNumMag))
            {
                this.boxNumMagToolTip.Active = true;

                // Remove a prior box num holdover.
                boxNumMagToolTip.SetToolTip(this.pictureBox2, "");
            }
        }
        /*
         * ***************
         */

        private void pictureBox1_MouseLeave(object sender, EventArgs e)
        {
            if ((ctClient != null) && (ctClient.boxSelClient == boxSelDest.boxNumMag))
            {
                this.boxNumMagToolTip.Active = false;
            }
        }

        private void pictureBox2_MouseLeave(object sender, EventArgs e)
        {
            if ((ctClient != null) && (ctClient.boxSelClient == boxSelDest.boxNumMag))
            {
                this.boxNumMagToolTip.Active = false;
            }
        }

        private void orphanBoxesBtn_Click(object sender, EventArgs e)
        {
            /*
             * Bring up the ToolBelt's Code Navigator tab if not
             * already up.
             */
            if (toolBelt == null)
            {
                ToolBeltStartUp(ToolBelt.tbTabPage.codeNav);
            }
            else if (toolBelt.tabPage != ToolBelt.tbTabPage.codeNav)
            {
                toolBelt.SelectTabPage(ToolBelt.tbTabPage.codeNav);
            }
        }
    }
}
